from UnifiedAI.api import API
from UnifiedAI.claude import Claude
from UnifiedAI.gpt import GPT
from UnifiedAI.gemini import Gemini


def AI(name : str, key : str, model : str) -> API:
	
	if "gpt" in model:
		return GPT(name, key, model)

	elif "claude" in model:
		return Claude(name, key, model)

	elif "gemini" in model:
		return Gemini(name, key, model)

	else:
		raise NameError("No model of that name.")



class Batch():
	def __init__(self, models: list):
		
		self.models = models

		self.usage : dict = {}


	def set_instructions(self,instructions : str) -> None:
		for model in self.models:
			model.set_instructions(instructions)


	def set_max_tokens(self, tokens : int) -> None:
		for model in self.models:
			model.set_max_tokens(tokens)


	def add_context(self, context : str) -> None:
		for model in self.models:
			model.add_context(context)


	def get_response(self, question : str) -> dict:

		responses = {}

		for model in self.models:
			responses[model.name] = model.get_response(question)

			self.usage[model.name] = model.Usage(model.usage.input_tokens,model.usage.output_tokens)

		return responses















from openai import OpenAI
from openai.types.chat import ChatCompletionMessageParam, ChatCompletionUserMessageParam
from UnifiedAI.api import API

class GPT(API):
	def __init__(self, name : str,  api_key: str, model : str):

		self.name = name

		self.type = "gpt"
		
		self.api_key = api_key

		self.connect = OpenAI(api_key=self.api_key)

		self.model_name = model

		self.system_instructions = "You are a helpful assistant."

		self.max_tokens = 512

		self.usage = self.Usage(0,0)

		self.api_calls = 0

		self.history: list[ChatCompletionMessageParam] = [{"role": "system", "content": f"{self.system_instructions}"},]



	def _trackUsage(self,message) -> None:

		self.api_calls += 1

		self.usage.input_tokens += message.usage.prompt_tokens

		self.usage.output_tokens += message.usage.completion_tokens


	def _ask(self) -> str:
		response = self.connect.chat.completions.create(
			model=self.model_name, messages=self.history,max_tokens=self.max_tokens)

		print(f"recieved {self.name}'s response.\n")

		self._trackUsage(response)

		return str(response.choices[0].message.content)

	def reset_history(self) -> None:
		self.history : list[ChatCompletionMessageParam] = [{"role": "system", "content": f"{self.system_instructions}"},]

	def _add(self, text: str) -> None:
		self.history.append(ChatCompletionUserMessageParam(
				role="user", content=f"{text}"))

	def set_instructions(self, instructions : str) -> None:
		self.system_instructions = instructions
		self.history[0] = {"role": "system", "content": f"{self.system_instructions}"}


	def history(self) -> list:
		
		history = []

		for item in self.history:
			history.append(f"{item['role']}: {item['content']}")  # type: ignore

		return history











from UnifiedAI.api import API
import google.generativeai as genai
from google.generativeai.types.generation_types import GenerateContentResponse


class Gemini(API):
	def __init__(self, name : str, api_key : str, model : str):
		
		self.name = name

		self.type = "gemini"

		self.api_key = api_key

		self.system_instructions =  "You are a helpful assistant."

		self.model_name = model

		self.max_tokens = 512

		genai.configure(api_key = self.api_key)

		self.connect = genai.GenerativeModel(
			model_name=self.model_name,
			generation_config = {"max_output_tokens": self.max_tokens},
			system_instruction = self.system_instructions,
        )

		self.usage = self.Usage(0,0)

		self.api_calls = 0

		self.history = []



	def _trackUsage(self,message) -> None:

		self.api_calls += 1

		self.usage.input_tokens += message.usage_metadata.prompt_token_count

		self.usage.output_tokens += message.usage_metadata.candidates_token_count


	def _ask(self):

		response : GenerateContentResponse = self.connect.start_chat(history=self.history).send_message("?")

		self._trackUsage(response)

		print(f"recieved {self.name}'s response.\n")

		return response.text


	def _add(self, text : str) -> None:
		self.history.append(
					{
						"role": "user",
						"parts": [f"{text}"],
					}
				)

	def reset_history(self) -> None:
		self.history = []

	def set_instructions(self, instructions : str) -> None:
		
		self.system_instructions = instructions

		self.connect = genai.GenerativeModel(
            model_name=self.model_name,
            system_instruction=self.system_instructions,
            generation_config = {"max_output_tokens": self.max_tokens}
        )


	def set_max_tokens(self,tokens: int) -> None:
		self.max_tokens = tokens

		self.connect = genai.GenerativeModel(
			model_name=self.model_name,
			system_instruction=self.system_instructions,
			generation_config = {"max_output_tokens": self.max_tokens}
		)

	def history(self) -> list:    
       
		history = []
        
		for item in self.history:
			history.append(f"{item['role']}: {item['parts'][0]}")

		return history






